import abjad
import pytest

import auxjad


def test_HarmonicNote_01():
    harm = auxjad.HarmonicNote(r"c''4")
    assert harm.style == "#'harmonic"
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic
        c''4
        """
    )


def test_HarmonicNote_02():
    harm1 = auxjad.HarmonicNote(r"c''4")
    harm2 = auxjad.HarmonicNote(r"c''", 1 / 4)
    harm3 = auxjad.HarmonicNote(12, 0.25)
    harm4 = auxjad.HarmonicNote(12, abjad.Duration(1, 4))
    harms = [harm1, harm2, harm3, harm4]
    for harm in harms:
        assert abjad.lilypond(harm) == abjad.String.normalize(
            r"""
            \tweak style #'harmonic
            c''4
            """
        )


def test_HarmonicNote_03():
    harm = auxjad.HarmonicNote(r"c''4",
                               style="#'harmonic-mixed",
                               )
    assert harm.style == "#'harmonic-mixed"
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic-mixed
        c''4
        """
    )


def test_HarmonicNote_04():
    harm = auxjad.HarmonicNote(r"c''4",
                               multiplier=(2, 3),
                               )
    assert harm.multiplier == abjad.Multiplier(2, 3)
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic
        c''4 * 2/3
        """
    )


def test_HarmonicNote_05():
    harm = auxjad.HarmonicNote(r"c''4")
    assert harm.written_pitch == "c''"
    assert harm.written_duration == 1 / 4
    assert harm.style == "#'harmonic"
    harm.written_pitch = 18
    harm.written_duration = abjad.Duration(1, 8)
    harm.style = "#'harmonic-mixed"
    assert harm.written_pitch == "fs''"
    assert harm.written_duration == 1 / 8
    assert harm.style == "#'harmonic-mixed"


def test_HarmonicNote_06():
    harm = auxjad.HarmonicNote(r"c''1",
                               style='flageolet',
                               )
    assert harm.style == 'flageolet'
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        c''1
        \flageolet
        """
    )


def test_HarmonicNote_07():
    harm1 = auxjad.HarmonicNote(r"d''1")
    harm2 = auxjad.HarmonicNote(r"d''1",
                                markup='III.',
                                )
    harm3 = auxjad.HarmonicNote(r"d''1",
                                markup='III.',
                                direction=abjad.Down)
    staff = abjad.Staff([harm1, harm2, harm3])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \tweak style #'harmonic
            d''1
            \once \override TextScript.parent-alignment-X = 0
            \once \override TextScript.self-alignment-X = 0
            \tweak style #'harmonic
            d''1
            ^ \markup { III. }
            \once \override TextScript.parent-alignment-X = 0
            \once \override TextScript.self-alignment-X = 0
            \tweak style #'harmonic
            d''1
            _ \markup { III. }
        }
        """
    )


def test_HarmonicNote_08():
    harm = auxjad.HarmonicNote(r"d''1",
                               markup='III.',
                               )
    harm.markup = None
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic
        d''1
        """
    )


def test_HarmonicNote_09():
    harm = auxjad.HarmonicNote(r"d''1")
    abjad.attach(abjad.Markup('test'), harm)
    harm.markup = 'III.'
    with pytest.raises(Exception):
        harm.markup = None


def test_HarmonicNote_10():
    harm = auxjad.HarmonicNote(r"d''1",
                               markup='III.',
                               direction=abjad.Down)
    assert harm.direction is abjad.Down
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        \tweak style #'harmonic
        d''1
        _ \markup { III. }
        """
    )
    harm.direction = abjad.Up
    assert harm.direction is abjad.Up
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        \tweak style #'harmonic
        d''1
        ^ \markup { III. }
        """
    )


def test_HarmonicNote_11():
    harm1 = auxjad.HarmonicNote(r"d''1")
    harm2 = auxjad.HarmonicNote(r"d''1",
                                markup='III.',
                                )
    harm3 = auxjad.HarmonicNote(r"d''1",
                                markup='III.',
                                centre_markup=True,
                                )
    harm4 = auxjad.HarmonicNote(r"d''1",
                                markup='III.',
                                centre_markup=False,
                                )
    assert abjad.lilypond(harm1) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic
        d''1
        """
    )
    assert abjad.lilypond(harm2) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        \tweak style #'harmonic
        d''1
        ^ \markup { III. }
        """
    )
    assert abjad.lilypond(harm3) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        \tweak style #'harmonic
        d''1
        ^ \markup { III. }
        """
    )
    assert abjad.lilypond(harm4) == abjad.String.normalize(
        r"""
        \tweak style #'harmonic
        d''1
        ^ \markup { III. }
        """
    )


def test_HarmonicNote_12():
    harm = auxjad.HarmonicNote(r"c''1",
                               style='flageolet',
                               )
    assert harm.style == 'flageolet'
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        c''1
        \flageolet
        """
    )
    harm = auxjad.HarmonicNote(r"c''1",
                               style=r'\flageolet',
                               )
    assert harm.style == r'\flageolet'
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        c''1
        \flageolet
        """
    )
    harm = auxjad.HarmonicNote(r"c''1",
                               style="#'flageolet",
                               )
    assert harm.style == "#'flageolet"
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        c''1
        \flageolet
        """
    )



import abjad
import pytest

import auxjad


def test_time_signature_list_01():
    container = abjad.Container(r"\time 3/4 c'2. \time 4/4 d'1")
    time_signatures = auxjad.get.time_signature_list(container)
    assert time_signatures == [abjad.TimeSignature((3, 4)),
                               abjad.TimeSignature((4, 4)),
                               ]


def test_time_signature_list_02():
    container = abjad.Container(r"\time 3/4 c'2. d'2. e'2.")
    time_signatures = auxjad.get.time_signature_list(container)
    assert time_signatures == [abjad.TimeSignature((3, 4)),
                               None,
                               None,
                               ]


def test_time_signature_list_03():
    container = abjad.Container(r"\time 5/8 c'4 ~ c'16 \time 3/8 d'4. e'4.")
    time_signatures = auxjad.get.time_signature_list(container)
    assert time_signatures == [abjad.TimeSignature((5, 8)),
                               abjad.TimeSignature((3, 8)),
                               None,
                               ]
    time_signatures = auxjad.get.time_signature_list(
        container,
        do_not_use_none=True,
    )
    assert time_signatures == [abjad.TimeSignature((5, 8)),
                               abjad.TimeSignature((3, 8)),
                               abjad.TimeSignature((3, 8)),
                               ]


def test_time_signature_list_04():
    container = abjad.Container(r"\time 3/4 c'2. d'2. \time 3/4 e'2. f'2.")
    time_signatures = auxjad.get.time_signature_list(container)
    assert time_signatures == [abjad.TimeSignature((3, 4)),
                               None,
                               abjad.TimeSignature((3, 4)),
                               None,
                               ]
    time_signatures = auxjad.get.time_signature_list(
        container,
        omit_repeated=True,
    )
    assert time_signatures == [abjad.TimeSignature((3, 4)),
                               None,
                               None,
                               None,
                               ]


def test_time_signature_list_05():
    container = abjad.Container(
        r"\time 3/4 c'2. d'2. \time 3/4 e'2. f'2."
    )
    with pytest.raises(ValueError):
        auxjad.get.time_signature_list(container,
                                       do_not_use_none=True,
                                       omit_repeated=True,
                                       )


def test_time_signature_list_06():
    container = abjad.Container(r"c'1 d'1 e'1 f'1")
    time_signatures = auxjad.get.time_signature_list(container)
    assert time_signatures == [abjad.TimeSignature((4, 4)),
                               None,
                               None,
                               None,
                               ]
    time_signatures = auxjad.get.time_signature_list(
        container,
        implicit_common_time=False,
    )
    assert time_signatures == [None,
                               None,
                               None,
                               None,
                               ]


def test_time_signature_list_07():
    container = abjad.Container(r"\time 4/4 c'1 d'1 e'1 f'1")
    time_signatures = auxjad.get.time_signature_list(
        container,
        do_not_use_none=True,
        implicit_common_time=False,
    )
    assert time_signatures == [abjad.TimeSignature((4, 4)),
                               abjad.TimeSignature((4, 4)),
                               abjad.TimeSignature((4, 4)),
                               abjad.TimeSignature((4, 4)),
                               ]
    container = abjad.Container(r"c'1 d'1 e'1 f'1")
    with pytest.raises(ValueError):
        auxjad.get.time_signature_list(container,
                                       do_not_use_none=True,
                                       implicit_common_time=False,
                                       )



import random

import abjad

import auxjad


def test_GeneticAlgorithmMusicMaker_01():
    random.seed(78612)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
    )
    assert repr(maker) == abjad.String.normalize(
        r"""
        pitches: ["c'", "d'", "e'", "f'"]
        attack_points: [0, 4, 8, 12]
        """
    )
    assert len(maker) == 4
    notes = maker.target_music
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = maker()
    assert maker.generation_number == 0
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            f'4
            ~
            f'8.
            e'16
            ~
            e'8
            a'8
        }
        """
    )
    notes = maker()
    assert maker.generation_number == 1
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'16
            e'8.
            ~
            e'4
            ~
            e'16
            g'8.
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_02():
    random.seed(48891)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            r16
            c'8.
            d'16
            d'8.
            f'4
            c'2
            d'16
            c''8.
            f'4
            r16
            c'8.
            d'4
            f'4.
            f'8
            c'4
            d'8.
            e'16
            ~
            e'4
            e'4
            c'4
            d'4
            ~
            d'16
            e'8.
            e'4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_03():
    random.seed(98119)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 8, 16, 24],
        attack_point_genes=list(range(32)),
        duration_unit=abjad.Duration((1, 32)),
        units_per_window=32,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            e'16
            ~
            e'16.
            e'32
            ~
            e'8
            ~
            e'32
            b'4...
            c'8.
            d'16
            ~
            d'2
            f'4
            c'4
            d'8..
            e'32
            ~
            e'4
            c'4
            c'4
            d'8..
            e'32
            ~
            e'4
            e'4
            c'4
            d'8..
            e'32
            ~
            e'4
            f'4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_04():
    random.seed(66732)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        duration_unit=abjad.Duration((1, 32)),
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            r16
            c'16
            d'16
            g'16
            ~
            g'8
            f'8
            c'32
            d'8..
            e'16
            f'8.
            c'8
            d'16
            e'16
            ~
            e'8
            f'8
            c'8
            d'16.
            e'32
            ~
            e'8
            f'8
            c'8
            d'16.
            e'32
            ~
            e'8
            f'8
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_05():
    random.seed(81242)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'", "g'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12, 16],
        attack_point_genes=list(range(20)),
        units_per_window=20,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            r8
            c'8
            d'8
            f'4.
            e'4..
            g'16
            c'8.
            d'16
            ~
            d'4
            ~
            d'16
            g'16
            f'8
            ~
            f'4
            g'4
            c'4
            d'8.
            e'16
            ~
            e'4
            a'4
            f'4
            c'4
            d'8.
            g'16
            ~
            g'4
            f'4
            g'4
            c'4
            d'4
            g'4
            f'4
            g'4
        }
        """
    )
    random.seed(81242)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'", "g'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12, 16],
        attack_point_genes=list(range(20)),
        units_per_window=20,
        omit_time_signature=True,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r8
            c'8
            d'8
            f'4.
            e'4..
            g'16
            c'8.
            d'4.
            g'16
            f'4.
            g'4
            c'4
            d'8.
            e'4
            ~
            e'16
            a'4
            f'4
            c'4
            d'8.
            g'4
            ~
            g'16
            f'4
            g'4
            c'4
            d'4
            g'4
            f'4
            g'4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_06():
    random.seed(22714)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        duration_unit=abjad.Duration((1, 32)),
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            r32
            c''16.
            ~
            c''32
            f'16.
            ~
            f'32
            e'16.
            f'8
            r32
            c'16.
            d'8
            ~
            d'16
            e'16
            e'8
            c'8.
            d'16
            e'8
            f'8
            r32
            c'16.
            d'8
            e'8
            f'8
            r32
            c'16.
            d'8
            e'8
            f'8
        }
        """
    )
    random.seed(22714)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        duration_unit=abjad.Duration((1, 32)),
        time_signatures=abjad.TimeSignature((2, 2))
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/2
            r32
            c''16.
            ~
            c''32
            f'16.
            ~
            f'32
            e'16.
            f'8
            r2
            r32
            c'16.
            d'8
            ~
            d'16
            e'16
            e'8
            r2
            c'8.
            d'16
            e'8
            f'8
            r2
            r32
            c'16.
            d'8
            e'8
            f'8
            r2
            r32
            c'16.
            d'8
            e'8
            f'8
            r2
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_07():
    random.seed(81242)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'", "g'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12, 16],
        attack_point_genes=list(range(20)),
        duration_unit=abjad.Duration((1, 16)),
        units_per_window=20,
        time_signatures=[abjad.TimeSignature((2, 4)),
                         abjad.TimeSignature((3, 4)),
                         ],
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            r8
            c'8
            d'8
            f'8
            ~
            \time 3/4
            f'4
            e'4..
            g'16
            \time 2/4
            c'8.
            d'16
            ~
            d'4
            ~
            \time 3/4
            d'16
            g'16
            f'4.
            g'4
            \time 2/4
            c'4
            d'8.
            e'16
            ~
            \time 3/4
            e'4
            a'4
            f'4
            \time 2/4
            c'4
            d'8.
            g'16
            ~
            \time 3/4
            g'4
            f'4
            g'4
            \time 2/4
            c'4
            d'4
            \time 3/4
            g'4
            f'4
            g'4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_08():
    random.seed(77132)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r16
            c'8
            d'16
            ~
            d'4
            f'4
            d'4
            r16
            c'8.
            d'8
            e'8
            ~
            e'8.
            f'16
            ~
            f'4
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        pitch_score_bias=0.95,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r8
            c'16
            d'16
            ~
            d'4
            ~
            d'8
            e'4
            d'8
            r8
            c'16
            d'16
            ~
            d'2
            ~
            d'16
            e'8.
            c'16
            d'8.
            ~
            d'8.
            e'16
            ~
            e'4
            f'4
            c'16
            d'8.
            ~
            d'8.
            e'16
            ~
            e'4
            f'4
            c'4
            ~
            c'16
            d'8
            e'16
            ~
            e'4
            f'4
        }
        """
    )
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        pitch_score_bias=0.05,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'16
            c''4..
            f'4
            c''4
            e'4
            d'4
            d'4
            f'4
            e'4
            d'4
            d'4
            f'4
            e'4
            d'4
            d'4
            f'4
            e'4
            d'4
            d'4
            f'4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_09():
    random.seed(87111)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", "d'", "e'", "f'"],
        pitch_genes=["c'", "d'", "e'", "f'", "g'", "a'", "b'", "c''"],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        attack_points_mode=True,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'16
            r8.
            c'16
            r16
            r16
            g'16
            r4
            f'16
            r8.
            c'16
            r8.
            d'16
            r8.
            e'16
            c'16
            r4.
            c'16
            r8.
            d'16
            r16
            r16
            e'16
            r4
            f'16
            r8.
            c'16
            r8.
            d'16
            r8.
            e'16
            r8.
            f'16
            r8.
            c'16
            r8.
            d'16
            r8.
            e'16
            r8.
            f'16
            r8.
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_10():
    random.seed(71441)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=["c'", None, "e'", ("g'", "bf'")],
        pitch_genes=[None,
                     "c'",
                     "d'",
                     "e'",
                     "f'",
                     "g'",
                     ("g'", "a'"),
                     ("g'", "bf'"),
                     ("g'", "c''"),
                     ],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        population_size=50,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <g' c''>4
            g'4
            ~
            g'16
            <g' c''>8.
            <g' c''>4
            c'2
            e'8.
            r16
            r4
            c'4
            r4
            r16
            e'8.
            <g' a'>4
            c'4
            r4
            e'8.
            <g' a'>16
            ~
            <g' a'>4
            c'4
            r4
            e'8.
            <g' a'>16
            ~
            <g' a'>4
        }
        """
    )


def test_GeneticAlgorithmMusicMaker_11():
    random.seed(71441)
    maker = auxjad.GeneticAlgorithmMusicMaker(
        pitch_target=[0, None, 4, (7, 10)],
        pitch_genes=[None, 0, 2, 4, 5, 7, (7, 9), (7, 10), (7, 12)],
        attack_point_target=[0, 4, 8, 12],
        attack_point_genes=list(range(16)),
        population_size=50,
    )
    notes = maker.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <g' c''>4
            g'4
            ~
            g'16
            <g' c''>8.
            <g' c''>4
            c'2
            e'8.
            r16
            r4
            c'4
            r4
            r16
            e'8.
            <g' a'>4
            c'4
            r4
            e'8.
            <g' a'>16
            ~
            <g' a'>4
            c'4
            r4
            e'8.
            <g' a'>16
            ~
            <g' a'>4
        }
        """
    )



import random

import abjad

import auxjad


def test_CrossFader_01():
    random.seed(17737)
    fade_out_container = abjad.Staff(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Staff(r"\times 4/5 {cs''4 d''1}")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    assert abjad.lilypond(fader) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            fs'4
            g'2
            bf'4
        }
        \new Staff
        {
            \times 4/5
            {
                \time 4/4
                cs''4
                d''1
            }
        }
        """
    )
    selection_a, selection_b = fader()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                fs'4
                g'2
                bf'4
            }
            \new Staff
            {
                \time 4/4
                R1
            }
        >>
        """
    )
    selection_a, selection_b = fader()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                fs'4
                g'2
                bf'4
            }
            \new Staff
            {
                \times 4/5
                {
                    \time 4/4
                    r4
                    d''1
                }
            }
        >>
        """
    )
    selection_a, selection_b = fader.current_window
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                fs'4
                g'2
                bf'4
            }
            \new Staff
            {
                \times 4/5
                {
                    \time 4/4
                    r4
                    d''1
                }
            }
        >>
        """
    )


def test_CrossFader_02():
    fade_out_container = abjad.Container(r"c'4 d'4 ~ d'4 r4")
    fade_in_container = abjad.Container(r"r2 c''2")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    assert len(fader) == 3
    fade_out_container = abjad.Container(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Container(r"\times 4/5 {cs''4 d''1}")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    assert len(fader) == 5
    fade_out_container = abjad.Container(r"c'4 d'4 ~ d'4 r4")
    fade_in_container = abjad.Container(r"r2 <c'' e'' g''>2")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    assert len(fader) == 5


def test_CrossFader_03():
    random.seed(44811)
    fade_out_container = abjad.Container(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Container(r"\times 4/5 {cs''4 d'1}")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    staff_a = abjad.Staff()
    staff_b = abjad.Staff()
    score = abjad.Score([staff_a, staff_b])
    for _ in range(3):
        selection_a, selection_b = fader()
        staff_a.extend(selection_a)
        staff_b.extend(selection_b)
    fader.reset()
    selection_a, selection_b = fader()
    staff_a.extend(selection_a)
    staff_b.extend(selection_b)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                fs'4
                g'2
                bf'4
                \time 4/4
                fs'4
                g'2
                bf'4
                \time 4/4
                fs'4
                r2
                bf'4
                \time 4/4
                fs'4
                g'2
                bf'4
            }
            \new Staff
            {
                \time 4/4
                R1
                \times 4/5
                {
                    \time 4/4
                    cs''4
                    r1
                }
                \times 4/5
                {
                    \time 4/4
                    cs''4
                    r1
                }
                \time 4/4
                R1
            }
        >>
        """
    )


def test_CrossFader_04():
    random.seed(44811)
    fade_out_container = abjad.Container(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Container(r"\times 4/5 {cs''4 d'1}")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                fs'4
                g'2
                bf'4
                fs'4
                g'2
                bf'4
                fs'4
                r2
                bf'4
                fs'4
                r2
                bf'4
                r2.
                bf'4
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                \times 4/5
                {
                    cs''4
                    r1
                }
                \times 4/5
                {
                    cs''4
                    r1
                }
                \times 4/5
                {
                    cs''4
                    d'1
                }
                \times 4/5
                {
                    cs''4
                    d'1
                }
                \times 4/5
                {
                    cs''4
                    d'1
                }
            }
        >>
        """
    )


def test_CrossFader_05():
    random.seed(10711)
    fade_out_container = abjad.Container(r"e'8 fs'4. r2")
    fade_in_container = abjad.Container(r"c''2 ~ c''8 d''4.")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_n(3)
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                e'8
                fs'4.
                r2
                e'8
                fs'4.
                r2
                e'8
                r2..
            }
            \new Staff
            {
                \time 4/4
                R1
                c''2
                ~
                c''8
                r4.
                c''2
                ~
                c''8
                r4.
            }
        >>
        """
    )


def test_CrossFader_06():
    random.seed(10711)
    fade_out_container = abjad.Container(r"e'8 fs'4. r2")
    fade_in_container = abjad.Container(r"c''2 ~ c''8 d''4.")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    staff_a = abjad.Staff()
    staff_b = abjad.Staff()
    score = abjad.Score([staff_a, staff_b])
    for selection_a, selection_b in fader:
        staff_a.extend(selection_a)
        staff_b.extend(selection_b)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                e'8
                fs'4.
                r2
                \time 4/4
                e'8
                fs'4.
                r2
                \time 4/4
                e'8
                r2..
                \time 4/4
                e'8
                r2..
                \time 4/4
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                \time 4/4
                c''2
                ~
                c''8
                r4.
                \time 4/4
                c''2
                ~
                c''8
                r4.
                \time 4/4
                c''2
                ~
                c''8
                d''4.
                \time 4/4
                c''2
                ~
                c''8
                d''4.
            }
        >>
        """
    )


def test_CrossFader_07():
    random.seed(87114)
    fade_out_container = abjad.Container(r"e'8 fs'4. r2")
    fade_in_container = abjad.Container(r"c''2 ~ c''8 d''4.")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    staff_a = abjad.Staff()
    staff_b = abjad.Staff()
    score = abjad.Score([staff_a, staff_b])
    for _ in range(3):
        selection_a, selection_b = next(fader)
        staff_a.extend(selection_a)
        staff_b.extend(selection_b)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                e'8
                fs'4.
                r2
                \time 4/4
                e'8
                r2..
                \time 4/4
                e'8
                r2..
            }
            \new Staff
            {
                \time 4/4
                R1
                \time 4/4
                R1
                \time 4/4
                c''2
                ~
                c''8
                r4.
            }
        >>
        """
    )


def test_CrossFader_08():
    random.seed(62190)
    fade_out_container = abjad.Container(r"\time 3/4 r4 c'4 d'4")
    fade_in_container = abjad.Container(r"\time 3/4 a''4 g''2")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                r4
                c'4
                d'4
                r2
                d'4
                r2
                d'4
                R1 * 3/4
                R1 * 3/4
            }
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                R1 * 3/4
                a''4
                r2
                a''4
                r2
                a''4
                g''2
            }
        >>
        """
    )
    random.seed(62190)
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_out_last=True,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                r4
                c'4
                d'4
                r2
                d'4
                r2
                d'4
                r2
                d'4
                R1 * 3/4
            }
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                R1 * 3/4
                a''4
                r2
                a''4
                g''2
                a''4
                g''2
            }
        >>
        """
    )
    random.seed(62190)
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_in_first=True,
                              fade_out_last=True,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                r4
                c'4
                d'4
                r4
                c'4
                d'4
                r4
                c'4
                d'4
                r2
                d'4
                R1 * 3/4
            }
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                a''4
                r2
                a''4
                g''2
                a''4
                g''2
                a''4
                g''2
            }
        >>
        """
    )


def test_CrossFader_09():
    random.seed(50137)
    fade_out_container = abjad.Container(r"e'2 c'2")
    fade_in_container = abjad.Container(
        r"c''8 d''8 e''8 f''8 g''8 a''8 b''8 c'''8"
    )
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                e'2
                c'2
                e'2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                R1
                R1
                R1
                R1
                R1
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                r4.
                f''8
                r2
                r4.
                f''8
                r2
                r4.
                f''8
                r8
                a''8
                r4
                r4.
                f''8
                r8
                a''8
                b''8
                r8
                r4.
                f''8
                r8
                a''8
                b''8
                r8
                r4.
                f''8
                r8
                a''8
                b''8
                c'''8
                r4
                e''8
                f''8
                r8
                a''8
                b''8
                c'''8
                r8
                d''8
                e''8
                f''8
                r8
                a''8
                b''8
                c'''8
                c''8
                d''8
                e''8
                f''8
                r8
                a''8
                b''8
                c'''8
                c''8
                d''8
                e''8
                f''8
                g''8
                a''8
                b''8
                c'''8
            }
        >>
        """
    )
    random.seed(50137)
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              weighted_duration=True,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                e'2
                c'2
                e'2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                r2
                c'2
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                r4.
                f''8
                r2
                r4.
                f''8
                r2
                r4.
                f''8
                r8
                a''8
                r4
                r4.
                f''8
                r8
                a''8
                b''8
                r8
                r4
                e''8
                f''8
                r8
                a''8
                b''8
                r8
                r4
                e''8
                f''8
                g''8
                a''8
                b''8
                r8
                c''8
                r8
                e''8
                f''8
                g''8
                a''8
                b''8
                r8
                c''8
                r8
                e''8
                f''8
                g''8
                a''8
                b''8
                c'''8
                c''8
                d''8
                e''8
                f''8
                g''8
                a''8
                b''8
                c'''8
                c''8
                d''8
                e''8
                f''8
                g''8
                a''8
                b''8
                c'''8
            }
        >>
        """
    )


def test_CrossFader_10():
    fade_out_container = abjad.Container(r"\time 3/4 e2 \times 2/3 {fs8 gs4}")
    fade_in_container = abjad.Container(r"\time 3/4 c'8 d' e' f' g' a'")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    fader.fade_out_contents = abjad.Container(r"\time 3/4 a4. bf4.")
    assert abjad.lilypond(fader) == abjad.String.normalize(
        r"""
        {
            %%% \time 3/4 %%%
            a4.
            bf4.
        }
        {
            %%% \time 3/4 %%%
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
        """
    )


def test_CrossFader_11():
    random.seed(41379)
    fade_out_container = abjad.Container(r"a'4 bf'2 r4")
    fade_in_container = abjad.Container(r"c''2 d''2")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              initial_repetitions=2,
                              final_repetitions=3,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                a'4
                bf'2
                r4
                a'4
                bf'2
                r4
                r4
                bf'2
                r4
                r4
                bf'2
                r4
                r4
                bf'2
                r4
                R1
                R1
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                R1
                R1
                r2
                d''2
                c''2
                d''2
                c''2
                d''2
                c''2
                d''2
                c''2
                d''2
            }
        >>
        """
    )


def test_CrossFader_12():
    random.seed(91766)
    fade_out_container = abjad.Container(r"a'4 bf'2 r4")
    fade_in_container = abjad.Container(r"c''2 d''2")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              repetition_chance=0.8,
                              )
    selection_a, selection_b = fader.output_n(4)
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                a'4
                bf'2
                r4
                a'4
                bf'2
                r4
                a'4
                bf'2
                r4
                a'4
                bf'2
                r4
            }
            \new Staff
            {
                \time 4/4
                R1
                r2
                d''2
                r2
                d''2
                r2
                d''2
            }
        >>
        """
    )


def test_CrossFader_13():
    random.seed(81943)
    fade_out_container = abjad.Container(
        r"\time 3/4 a'4 bf'2 ~ \time 2/4 bf'4 f'4"
    )
    fade_in_container = abjad.Container(
        r"\time 3/4 r16 cs''4.. e''4 \time 2/4 d''2"
    )
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_n(3)
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                a'4
                bf'2
                ~
                \time 2/4
                bf'4
                f'4
                \time 3/4
                a'4
                bf'2
                ~
                \time 2/4
                bf'4
                f'4
                \time 3/4
                a'4
                bf'2
                ~
                \time 2/4
                bf'4
                r4
            }
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                \time 2/4
                R1 * 1/2
                \time 3/4
                r16
                cs''4..
                r4
                \time 2/4
                R1 * 1/2
                \time 3/4
                r16
                cs''4..
                r4
                \time 2/4
                R1 * 1/2
            }
        >>
        """
    )


def test_CrossFader_14():
    random.seed(75991)
    fade_out_container = abjad.Container(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Container(r"\times 4/5 {cs''4 d''1}")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              omit_time_signatures=True,
                              )
    selection_a, selection_b = fader.output_n(3)
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                fs'4
                g'2
                bf'4
                fs'4
                g'2
                r4
                fs'4
                g'2
                r4
            }
            \new Staff
            {
                R1
                R1
                \times 4/5
                {
                    cs''4
                    r1
                }
            }
        >>
        """
    )


def test_CrossFader_15():
    random.seed(33163)
    fade_out_container = abjad.Container(r"c'8 d'4 e'8 ~ e'2")
    fade_in_container = abjad.Container(r"c'2 d'2")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              disable_rewrite_meter=True,
                              )
    selection_a, selection_b = fader.output_n(3)
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                c'8
                d'4
                e'8
                ~
                e'2
                r8
                d'4
                e'8
                ~
                e'2
                r8
                d'4
                e'8
                ~
                e'2
            }
            \new Staff
            {
                \time 4/4
                R1
                R1
                r2
                d'2
            }
        >>
        """
    )


def test_CrossFader_16():
    random.seed(81662)
    fade_out_container = abjad.Container(r"\time 3/4 c'4 d'4 e'4")
    fade_in_container = abjad.Container(r"\time 4/4 g'2 a'2")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_in_first=True,
                              fade_out_last=True,
                              weighted_duration=True,
                              )
    selection_a, selection_b = fader.output_all()
    staff_a = abjad.Staff(selection_a)
    staff_b = abjad.Staff(selection_b)
    auxjad.mutate.sync_containers([staff_a, staff_b])
    score = abjad.Score([staff_a, staff_b])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                c'4
                d'4
                e'4
                c'4
                d'4
                e'4
                c'4
                r4
                e'4
                c'4
                r2
                c'4
                r2
                R1 * 3/4
                R1 * 3/4
                R1 * 3/4
            }
            \new Staff
            {
                \time 4/4
                R1
                r2
                a'2
                r2
                a'2
                r2
                a'2
                g'2
                a'2
                g'2
                a'2
            }
        >>
        """
    )


def test_CrossFader_17():
    fade_out_container = abjad.Container(r"fs'4 g'2 bf'4")
    fade_in_container = abjad.Container(r"\times 4/5 {cs''4 d''1}")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_in_first=True,
                              fade_out_last=True,
                              initial_repetitions=3,
                              final_repetitions=3,
                              repetition_chance=0.7,
                              weighted_duration=True,
                              disable_rewrite_meter=True,
                              omit_time_signatures=True,
                              use_multimeasure_rests=True,
                              boundary_depth=True,
                              maximum_dot_count=True,
                              rewrite_tuplets=True,
                              )
    assert fader.fade_in_first
    assert fader.fade_out_last
    assert fader.initial_repetitions == 3
    assert fader.final_repetitions == 3
    assert fader.repetition_chance == 0.7
    assert fader.weighted_duration
    assert fader.disable_rewrite_meter
    assert fader.omit_time_signatures
    assert fader.use_multimeasure_rests
    assert fader.boundary_depth
    assert fader.maximum_dot_count
    assert fader.rewrite_tuplets
    fader.fade_in_first = False
    fader.fade_out_last = False
    fader.initial_repetitions = 4
    fader.final_repetitions = 7
    fader.repetition_chance = 0.23
    fader.weighted_duration = False
    fader.disable_rewrite_meter = False
    fader.omit_time_signatures = False
    fader.use_multimeasure_rests = False
    fader.boundary_depth = False
    fader.maximum_dot_count = False
    fader.rewrite_tuplets = False
    assert not fader.fade_in_first
    assert not fader.fade_out_last
    assert fader.initial_repetitions == 4
    assert fader.final_repetitions == 7
    assert fader.repetition_chance == 0.23
    assert not fader.weighted_duration
    assert not fader.disable_rewrite_meter
    assert not fader.omit_time_signatures
    assert not fader.use_multimeasure_rests
    assert not fader.boundary_depth
    assert not fader.maximum_dot_count
    assert not fader.rewrite_tuplets


def test_CrossFader_18():
    random.seed(97142)
    fade_out_container = abjad.Container(r"c'4.\p e'8--\f ~ e'2")
    fade_in_container = abjad.Container(
        r"\times 2/3 {f'4-.\pp r4 d'4->\f ~ } d'2"
    )
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_in_first=True,
                              fade_out_last=True,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 4/4
                c'4.
                \p
                e'8
                \f
                - \tenuto
                ~
                e'2
                c'4.
                \p
                e'8
                \f
                - \tenuto
                ~
                e'2
                r4.
                e'8
                - \tenuto
                ~
                e'2
                r4.
                e'8
                - \tenuto
                ~
                e'2
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                \times 2/3
                {
                    f'4
                    \pp
                    - \staccato
                    r2
                }
                r2
                \times 2/3
                {
                    f'4
                    - \staccato
                    r2
                }
                r2
                \times 2/3
                {
                    f'4
                    - \staccato
                    r4
                    d'4
                    \f
                    - \accent
                    ~
                }
                d'2
                \times 2/3
                {
                    f'4
                    \pp
                    - \staccato
                    r4
                    d'4
                    \f
                    - \accent
                    ~
                }
                d'2
            }
        >>
        """
    )


def test_CrossFader_19():
    random.seed(25519)
    fade_out_container = abjad.Container(r"\times 2/3 {<c' e'>2 g'1}")
    fade_in_container = abjad.Container(r"<d' ef'>2. <bf a'>4")
    fader = auxjad.CrossFader(fade_out_container,
                              fade_in_container,
                              fade_in_first=True,
                              fade_out_last=True,
                              )
    selection_a, selection_b = fader.output_all()
    score = abjad.Score([
        abjad.Staff(selection_a),
        abjad.Staff(selection_b),
    ])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \times 2/3
                {
                    \time 4/4
                    <c' e'>2
                    g'1
                }
                \times 2/3
                {
                    <c' e'>2
                    g'1
                }
                \times 2/3
                {
                    <c' e'>2
                    g'1
                }
                \times 2/3
                {
                    <c' e'>2
                    g'1
                }
                \times 2/3
                {
                    c'2
                    g'1
                }
                \times 2/3
                {
                    c'2
                    g'1
                }
                \times 2/3
                {
                    c'2
                    r1
                }
                R1
            }
            \new Staff
            {
                \time 4/4
                R1
                ef'2.
                r4
                <d' ef'>2.
                r4
                <d' ef'>2.
                bf4
                <d' ef'>2.
                bf4
                <d' ef'>2.
                <bf a'>4
                <d' ef'>2.
                <bf a'>4
                <d' ef'>2.
                <bf a'>4
            }
        >>
        """
    )


def test_CrossFader_20():
    random.seed(87144)
    fade_out_container = abjad.Container(r"b'8 c''8 e''2 g''4")
    fade_in_container = abjad.Container(r"\times 2/3 {e'2 d'2 c'2}")
    fader = auxjad.CrossFader(fade_out_container, fade_in_container)
    selection_a, selection_b = fader.output_all()
    literal_voice_one = abjad.LilyPondLiteral(r'\voiceOne')
    literal_voice_two = abjad.LilyPondLiteral(r'\voiceTwo')
    abjad.attach(literal_voice_one, selection_a[0])
    abjad.attach(literal_voice_two, selection_b[0])
    staff = abjad.Staff(
        [abjad.Voice(selection_a), abjad.Voice(selection_b)],
        simultaneous=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        <<
            \new Voice
            {
                \time 4/4
                \voiceOne
                b'8
                c''8
                e''2
                g''4
                b'8
                c''8
                e''2
                r4
                b'8
                c''8
                e''2
                r4
                b'8
                c''8
                e''2
                r4
                b'8
                c''8
                e''2
                r4
                b'8
                c''8
                r2.
                b'8
                r2..
                R1
            }
            \new Voice
            {
                \time 4/4
                \voiceTwo
                R1
                R1
                \times 2/3
                {
                    r1
                    c'2
                }
                \times 2/3
                {
                    e'2
                    r2
                    c'2
                }
                \times 2/3
                {
                    e'2
                    d'2
                    c'2
                }
                \times 2/3
                {
                    e'2
                    d'2
                    c'2
                }
                \times 2/3
                {
                    e'2
                    d'2
                    c'2
                }
                \times 2/3
                {
                    e'2
                    d'2
                    c'2
                }
            }
        >>
        """
    )



import abjad

import auxjad


def test_reposition_clefs_01():
    staff = abjad.Staff(r"c'1 | d'1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('treble'), staff[1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            \clef "treble"
            d'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | d'1")
    abjad.attach(abjad.Clef('treble'), staff[1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "treble"
            d'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'1
        }
        """
    )


def test_reposition_clefs_02():
    staff = abjad.Staff(r"c'1 | d'2 e'4 r4 | f'1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            e'4
            r4
            \clef "treble"
            f'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            e'4
            r4
            f'1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | d'2 e'4 r4 | f'1")
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            e'4
            r4
            \clef "treble"
            f'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            e'4
            r4
            f'1
        }
        """
    )


def test_reposition_clefs_03():
    staff = abjad.Staff(r"c'1 | a,2 bf,4 r4 | f'1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('bass'), staff[1])
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            \clef "bass"
            a,2
            bf,4
            r4
            \clef "treble"
            f'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            \clef "bass"
            a,2
            bf,4
            r4
            \clef "treble"
            f'1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | a,2 bf,4 r4 | f'1")
    abjad.attach(abjad.Clef('bass'), staff[1])
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "bass"
            a,2
            bf,4
            r4
            \clef "treble"
            f'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "bass"
            a,2
            bf,4
            r4
            \clef "treble"
            f'1
        }
        """
    )


def test_reposition_clefs_04():
    staff = abjad.Staff(r"c'1 | d'2 r2 | R1 | e'1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            r2
            R1
            \clef "treble"
            e'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            r2
            R1
            e'1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | d'2 r2 | R1 | e'1")
    abjad.attach(abjad.Clef('treble'), staff[4])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            r2
            R1
            \clef "treble"
            e'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            r2
            R1
            e'1
        }
        """
    )


def test_reposition_clefs_05():
    staff = abjad.Staff(r"c'1 | d'2 r2 | fs1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('bass'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            r2
            \clef "bass"
            fs1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | d'2 r2 | fs1")
    abjad.attach(abjad.Clef('bass'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            r2
            \clef "bass"
            fs1
        }
        """
    )


def test_reposition_clefs_06():
    staff = abjad.Staff(r"c'1 | d'2 r2 | fs1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('bass'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:], shift_clef_to_notes=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )
    staff = abjad.Staff(r"c'1 | d'2 r2 | fs1")
    abjad.attach(abjad.Clef('bass'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:], shift_clef_to_notes=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'2
            \clef "bass"
            r2
            fs1
        }
        """
    )


def test_reposition_clefs_07():
    staff = abjad.Staff([abjad.Note("c'2"),
                         abjad.Chord("<d' f'>2"),
                         abjad.Tuplet((2, 3), "g'2 a'2 b'2"),
                         ])
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('treble'), staff[2][1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'2
            <d' f'>2
            \times 2/3
            {
                g'2
                \clef "treble"
                a'2
                b'2
            }
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'2
            <d' f'>2
            \times 2/3
            {
                g'2
                a'2
                b'2
            }
        }
        """
    )
    staff = abjad.Staff([abjad.Note("c'2"),
                         abjad.Chord("<d' f'>2"),
                         abjad.Tuplet((2, 3), "g'2 a'2 b'2"),
                         ])
    abjad.attach(abjad.Clef('treble'), staff[2][1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            <d' f'>2
            \times 2/3
            {
                g'2
                \clef "treble"
                a'2
                b'2
            }
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            <d' f'>2
            \times 2/3
            {
                g'2
                a'2
                b'2
            }
        }
        """
    )


def test_reposition_clefs_08():
    staff = abjad.Staff(r"c'1 | d'1")
    abjad.attach(abjad.Clef('treble'), staff[1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "treble"
            d'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'1
        }
        """
    )


def test_reposition_clefs_09():
    staff = abjad.Staff(r"c1 | d1")
    abjad.attach(abjad.Clef('bass'), staff[1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c1
            \clef "bass"
            d1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:], implicit_clef=abjad.Clef('bass'))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c1
            d1
        }
        """
    )


def test_reposition_clefs_10():
    staff = abjad.Staff(r"c'1 | r1 | d'1")
    abjad.attach(abjad.Clef('bass'), staff[1])
    abjad.attach(abjad.Clef('treble'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "bass"
            r1
            \clef "treble"
            d'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:], shift_clef_to_notes=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \clef "bass"
            r1
            \clef "treble"
            d'1
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            r1
            d'1
        }
        """
    )


def test_reposition_clefs_11():
    staff = abjad.Staff(r"\time 3/4 c'2. | d'4 r2 | R1 * 3/4 | e'2.")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('bass'), staff[2])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            \clef "treble"
            c'2.
            d'4
            \clef "bass"
            r2
            R1 * 3/4
            e'2.
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:], shift_clef_to_notes=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            \clef "treble"
            c'2.
            d'4
            \clef "bass"
            r2
            R1 * 3/4
            e'2.
        }
        """
    )
    auxjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            \clef "treble"
            c'2.
            d'4
            r2
            R1 * 3/4
            \clef "bass"
            e'2.
        }
        """
    )


def test_reposition_clefs_12():
    staff = abjad.Staff(r"c'1 | d'1")
    abjad.attach(abjad.Clef('treble'), staff[0])
    abjad.attach(abjad.Clef('treble'), staff[1])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            \clef "treble"
            d'1
        }
        """
    )
    abjad.mutate.reposition_clefs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "treble"
            c'1
            d'1
        }
        """
    )



import abjad

import auxjad


def test_merge_partial_tuplets_01():
    staff = abjad.Staff(r"\times 2/3 {c'1} \times 2/3 {d'2}")
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'1
                d'2
            }
        }
        """
    )


def test_merge_partial_tuplets_02():
    staff = abjad.Staff(
        r"\times 2/3 {r4} \times 2/3 {c'2} "
        r"\times 4/5 {d'2~} \times 4/5{d'8}"
    )
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                r4
                c'2
            }
            \times 4/5
            {
                d'2
                ~
                d'8
            }
        }
        """
    )


def test_merge_partial_tuplets_03():
    staff = abjad.Staff(r"\times 2/3 {c'2} \times 2/3 {d'2} \times 2/3 {e'2}")
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'2
                d'2
                e'2
            }
        }
        """
    )


def test_merge_partial_tuplets_04():
    staff = abjad.Staff(r"\times 2/3 {c'2\p\< d'2} \times 2/3 {e'2\ff}")
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'2
                \p
                \<
                d'2
                e'2
                \ff
            }
        }
        """
    )


def test_merge_partial_tuplets_05():
    staff = abjad.Staff(
        r"\times 2/3 {c'4 d'4 e'4} \times 2/3 {f'4} \times 2/3 {g'4 a'4}"
    )
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'4
                d'4
                e'4
            }
            \times 2/3
            {
                f'4
                g'4
                a'4
            }
        }
        """
    )


def test_merge_partial_tuplets_06():
    staff = abjad.Staff(
        r"\time 3/4 c'2. "
        r"\times 2/3 {d'4} r4 \times 2/3 {e'2} "
        r"\times 2/3 {f'4} r4 \times 2/3 {g'2}"
    )
    auxjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                d'4
            }
            r4
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                e'2
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                f'4
            }
            r4
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                g'2
            }
        }
        """
    )
    staff = abjad.Staff(
        r"\time 3/4 c'2. "
        r"\times 2/3 {d'4} r4 \times 2/3 {e'2} "
        r"\times 2/3 {f'4} r4 \times 2/3 {g'2}"
    )
    auxjad.mutate.merge_partial_tuplets(staff[:], merge_across_barlines=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                d'4
            }
            r4
            \times 2/3
            {
                e'2
                f'4
            }
            r4
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                g'2
            }
        }
        """
    )


def test_merge_partial_tuplets_07():
    staff = abjad.Staff(r"\times 2/3 {c'1} \times 2/3 {d'2}")
    abjad.mutate.merge_partial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'1
                d'2
            }
        }
        """
    )



import random

import abjad
import pytest

import auxjad


def test_WindowLooper_01():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert abjad.lilypond(looper) == abjad.String.normalize(
        r"""
        {
            c'4
            d'2
            e'4
            f'2
            ~
            f'8
            g'4.
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'2
            e'4
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'4
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )
    notes = looper.current_window
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'4
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )


def test_WindowLooper_02():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 8),
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'8
            ~
            d'4.
            e'8
        }
        """
    )


def test_WindowLooper_03():
    container = abjad.Container(r"c'4 d'2 e'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 8),
                                 )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'8
            ~
            d'4.
            e'8
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'2
            e'4
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4.
            e'4
            r8
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4
            e'4
            r4
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8
            e'4
            r4.
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'4
            r2
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'8
            r8
            r2
        }
        """
    )
    with pytest.raises(StopIteration):
        notes = looper.__next__()  # noqa: F841


def test_WindowLooper_04():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(5, 8),
                                 max_steps=2,
                                 repetition_chance=0.25,
                                 forward_bias=0.2,
                                 head_position=(2, 8),
                                 omit_time_signatures=False,
                                 fill_with_rests=False,
                                 boundary_depth=0,
                                 maximum_dot_count=1,
                                 rewrite_tuplets=False,
                                 process_on_first_call=True,
                                 )
    assert looper.window_size == abjad.Meter((3, 4))
    assert looper.step_size == abjad.Duration((5, 8))
    assert looper.max_steps == 2
    assert looper.repetition_chance == 0.25
    assert looper.forward_bias == 0.2
    assert looper.head_position == abjad.Duration((1, 4))
    assert not looper.omit_time_signatures
    assert not looper.fill_with_rests
    assert looper.boundary_depth == 0
    assert looper.maximum_dot_count == 1
    assert not looper.rewrite_tuplets
    assert looper.process_on_first_call
    looper.window_size = (5, 4)
    looper.step_size = (1, 4)
    looper.max_steps = 3
    looper.repetition_chance = 0.1
    looper.forward_bias = 0.8
    looper.head_position = 0
    looper.omit_time_signatures = True
    looper.fill_with_rests = True
    looper.boundary_depth = 1
    looper.maximum_dot_count = 2
    looper.rewrite_tuplets = True
    looper.process_on_first_call = False
    assert looper.window_size == abjad.Meter((5, 4))
    assert looper.step_size == abjad.Duration((1, 4))
    assert looper.max_steps == 3
    assert looper.repetition_chance == 0.1
    assert looper.forward_bias == 0.8
    assert looper.head_position == abjad.Duration(0)
    assert looper.omit_time_signatures
    assert looper.fill_with_rests
    assert looper.boundary_depth == 1
    assert looper.maximum_dot_count == 2
    assert looper.rewrite_tuplets
    assert not looper.process_on_first_call


def test_WindowLooper_05():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            d'4
            e'4
            f'4
            e'4
            f'4
            r4
            f'4
            r2
        }
        """
    )


def test_WindowLooper_06():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'2
            e'4
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'4
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            ~
            d'4
            ~
            d'8
            e'4
            f'8
        }
        """
    )
    looper.window_size = (3, 8)
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'16
            d'16
            ~
            d'4
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            d'4.
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            d'4.
        }
        """
    )


def test_WindowLooper_07():
    container = abjad.Container(r"\times 2/3 {c'8 d'8 e'8} d'2.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 16),
                                 )
    notes = looper.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 3/4
                c'8
                d'8
                e'8
            }
            d'2
            \times 2/3
            {
                c'32
                d'16
                ~
                d'16
                e'8
            }
            d'16
            ~
            d'2
            \times 2/3
            {
                d'16
                e'8
            }
            d'8
            ~
            d'2
        }
        """
    )


def test_WindowLooper_08():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 omit_time_signatures=True,
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'2
            e'4
        }
        """
    )


def test_WindowLooper_09():
    wrong_type_input = 'foobar'
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    with pytest.raises(TypeError):
        looper = auxjad.WindowLooper(wrong_type_input,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=17j,
                                     step_size=(1, 16),
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=True,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     max_steps='foo',
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     repetition_chance='bar',
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     forward_bias=False,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     head_position=62.3j,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     omit_time_signatures='xyz',
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     fill_with_rests=1.2,
                                     )
    with pytest.raises(ValueError):
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(100, 1),
                                     step_size=(1, 16),
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     max_steps=-1,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     repetition_chance=-0.3,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     repetition_chance=1.4,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     forward_bias=-0.3,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     forward_bias=1.4,
                                     )
        looper = auxjad.WindowLooper(container,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     head_position=(100, 1),
                                     )


def test_WindowLooper_10():
    container = abjad.Container(r"c'4 e'2 d'2 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_all(tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            e'2
            ~
            e'2
            d'4
            e'4
            d'2
            ~
            d'2
            f'4
            d'4
            f'4
            r4
            f'4
            r2
        }
        """
    )


def test_WindowLooper_11():
    container = abjad.Container(r"c'4 <e' f' g'>2 r4 f'2.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_all(tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            <e' f' g'>2
            ~
            <e' f' g'>2
            r4
            <e' f' g'>4
            r4
            f'4
            r4
            f'2
            ~
            f'2.
            ~
            f'2
            r4
            f'4
            r2
        }
        """
    )


def test_WindowLooper_12():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_WindowLooper_13():
    container = abjad.Container(r"c'4 d'2 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_n(2, tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
            ~
            d'2
            e'4
        }
        """
    )


def test_WindowLooper_14():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 )
    with pytest.raises(RuntimeError):
        notes = looper.output_n(100)  # noqa: F841


def test_WindowLooper_15():
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 head_position=(3, 4),
                                 forward_bias=0.0,
                                 )
    notes = looper.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            f'4
            g'4
            a'4
            e'4
            f'4
            g'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_WindowLooper_16():
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 head_position=(3, 4),
                                 forward_bias=0.0,
                                 )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            f'4
            g'4
            a'4
            e'4
            f'4
            g'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
        }
        """
    )


def test_WindowLooper_17():
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 forward_bias=0.0,
                                 )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
        }
        """
    )


def test_WindowLooper_18():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 process_on_first_call=True,
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'4
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )


def test_WindowLooper_19():
    container = abjad.Container(r"c'4-.\p\< d'2--\f e'4->\ppp f'2 ~ f'8")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    staff = abjad.Staff()
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            \p
            - \staccato
            \<
            d'2
            \f
            - \tenuto
            e'4
            \ppp
            - \accent
            c'8.
            \p
            - \staccato
            \<
            d'16
            \f
            - \tenuto
            ~
            d'4
            ~
            d'8.
            e'16
            \ppp
            - \accent
            ~
            e'8.
            f'16
        }
        """
    )


def test_WindowLooper_20():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'2
            e'4
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'4
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )
    looper.contents = abjad.Container(r"c'16 d'16 e'16 f'16 g'2. | a'1")
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'16
            e'16
            f'16
            g'16
            ~
            g'2
            ~
            g'8.
            a'16
        }
        """
    )
    looper.head_position = 0
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'16
            d'16
            e'16
            f'16
            g'2.
        }
        """
    )


def test_WindowLooper_21():
    container = abjad.Container(r"c'1")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert len(looper) == 16
    container = abjad.Container(r"c'1")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 4),
                                 )
    assert len(looper) == 4
    container = abjad.Container(r"c'2..")
    looper = auxjad.WindowLooper(container,
                                 window_size=(2, 4),
                                 step_size=(1, 4),
                                 )
    assert len(looper) == 4


def test_WindowLooper_22():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 fill_with_rests=False,
                                 )
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_WindowLooper_23():
    random.seed(43271)
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 head_position=(3, 4),
                                 forward_bias=0.5,
                                 )
    notes = looper.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            f'4
            g'4
            a'4
            e'4
            f'4
            g'4
            d'4
            e'4
            f'4
            e'4
            f'4
            g'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_WindowLooper_24():
    random.seed(81723)
    container = abjad.Container(
        r"c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4 d''4 e''4 f''4"
    )
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 4),
                                 max_steps=4,
                                 )
    notes = looper.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            e'4
            f'4
            g'4
            f'4
            g'4
            a'4
            c''4
            d''4
            e''4
        }
        """
    )


def test_WindowLooper_25():
    container = abjad.Container(r"c'4. d'8 e'2")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'2
        }
        """
    )
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 boundary_depth=1,
                                 )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            e'2
        }
        """
    )


def test_WindowLooper_26():
    container = abjad.Container(r"c'4 d'2 e'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 8),
                                 )
    staff = abjad.Staff()
    for window in looper:
        staff.append(window)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
            \time 3/4
            c'8
            d'8
            ~
            d'4.
            e'8
            \time 3/4
            d'2
            e'4
            \time 3/4
            d'4.
            e'4
            r8
            \time 3/4
            d'4
            e'4
            r4
            \time 3/4
            d'8
            e'4
            r4.
            \time 3/4
            e'4
            r2
            \time 3/4
            e'8
            r8
            r2
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
            c'8
            d'8
            ~
            d'4.
            e'8
            d'2
            e'4
            d'4.
            e'4
            r8
            d'4
            e'4
            r4
            d'8
            e'4
            r4.
            e'4
            r2
            e'8
            r8
            r2
        }
        """
    )


def test_WindowLooper_27():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    looper = auxjad.WindowLooper(tuplet,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(voice,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    looper = auxjad.WindowLooper(staff,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    looper = auxjad.WindowLooper(score,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    looper = auxjad.WindowLooper(staff,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    looper = auxjad.WindowLooper(score,
                                 window_size=(4, 4),
                                 step_size=(1, 16),
                                 )
    assert isinstance(looper(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        looper = auxjad.WindowLooper(staff,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     )

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        looper = auxjad.WindowLooper(score,  # noqa: F841
                                     window_size=(4, 4),
                                     step_size=(1, 16),
                                     )


def test_WindowLooper_28():
    container = abjad.Container(r"c'4.\p( d'8 e'8\f) f'4.\p( ~ f'4 g'1\pp)")
    looper = auxjad.WindowLooper(container,
                                 window_size=(4, 4),
                                 step_size=(1, 4),
                                 )
    notes = looper.output_n(6)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            \p
            (
            d'8
            e'8
            \f
            )
            f'4.
            \p
            c'8
            (
            d'8
            e'8
            \f
            )
            f'8
            \p
            ~
            f'2
            e'8
            \f
            f'8
            \p
            (
            ~
            f'2
            g'4
            \pp
            )
            f'2
            \p
            (
            g'2
            \pp
            )
            f'4
            \p
            (
            g'2.
            \pp
            )
            g'1
        }
        """
    )


def test_WindowLooper_29():
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 16),
                                 )
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
        }
        """
    )
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4")
    looper = auxjad.WindowLooper(container,
                                 window_size=(3, 4),
                                 step_size=(1, 16),
                                 disable_rewrite_meter=True,
                                 )
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            c'8.
            d'4
            e'4
            f'16
        }
        """
    )



import random

import abjad
import pytest

import auxjad


def test_Phaser_01():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    assert abjad.lilypond(phaser) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )
    notes = phaser.current_window
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )


def test_Phaser_02():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'4
            e'8
            ~
            e'8
            f'4
            c'8
        }
        """
    )


def test_Phaser_03():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 ~ e'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            ~
            e'2.
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4
            e'2
            ~
            e'2
            c'4
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'2.
            ~
            e'4
            c'4
            d'4
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'2.
            c'4
            d'4
            e'4
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'2
            c'4
            d'4
            e'2
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'4
            c'4
            d'4
            e'2.
        }
        """
    )
    with pytest.raises(StopIteration):
        notes = phaser.__next__()  # noqa: F841


def test_Phaser_04():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(5, 8),
                           max_steps=2,
                           forward_bias=0.2,
                           remove_unterminated_ties=True,
                           omit_time_signatures=True,
                           boundary_depth=0,
                           maximum_dot_count=1,
                           rewrite_tuplets=False,
                           process_on_first_call=True,
                           )
    assert phaser.step_size == abjad.Duration((5, 8))
    assert phaser.max_steps == 2
    assert phaser.forward_bias == 0.2
    assert phaser.remove_unterminated_ties
    assert phaser.boundary_depth == 0
    assert phaser.omit_time_signatures
    assert phaser.maximum_dot_count == 1
    assert not phaser.rewrite_tuplets
    assert phaser.process_on_first_call
    phaser.step_size = (1, 4)
    phaser.max_steps = 3
    phaser.forward_bias = 0.8
    phaser.remove_unterminated_ties = False
    phaser.omit_time_signatures = False
    phaser.boundary_depth = 1
    phaser.maximum_dot_count = 2
    phaser.rewrite_tuplets = True
    phaser.process_on_first_call = False
    assert phaser.step_size == abjad.Duration((1, 4))
    assert phaser.max_steps == 3
    assert phaser.forward_bias == 0.8
    assert not phaser.remove_unterminated_ties
    assert not phaser.omit_time_signatures
    assert phaser.boundary_depth == 1
    assert phaser.maximum_dot_count == 2
    assert phaser.rewrite_tuplets
    assert not phaser.process_on_first_call


def test_Phaser_05():
    container = abjad.Container(r"\time 3/4 c'4. d'4.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    notes = phaser.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4.
            d'4.
            c'8
            d'4.
            c'4
            d'4
            c'4.
            d'8
            c'4.
            d'4.
        }
        """
    )
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    notes = phaser.output_all(cycle_back_to_first=False)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4.
            d'4.
            c'8
            d'4.
            c'4
            d'4
            c'4.
            d'8
        }
        """
    )


def test_Phaser_06():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 ~ e'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            ~
            e'2.
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4
            e'2
            ~
            e'2
            c'4
        }
        """
    )
    phaser.step_size = (1, 16)
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8.
            e'16
            ~
            e'2
            ~
            e'4..
            c'16
            ~
            c'8.
            d'16
        }
        """
    )
    notes = phaser.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8
            e'8
            ~
            e'2
            ~
            e'4.
            c'4
            d'8
        }
        """
    )


def test_Phaser_07():
    container = abjad.Container(r"\times 2/3 {c'8 d'8 e'8} d'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    notes = phaser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 4/4
                c'8
                d'8
                e'8
            }
            d'2.
            \times 2/3
            {
                c'32
                d'16
                ~
                d'16
                e'8
            }
            d'16
            ~
            d'2
            ~
            d'8.
            c'16
            \times 2/3
            {
                d'16
                e'8
            }
            d'8
            ~
            d'2
            ~
            d'8
            \times 2/3
            {
                c'8
                d'16
            }
        }
        """
    )


def test_Phaser_08():
    wrong_type_input = 'foobar'
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    with pytest.raises(TypeError):
        phaser = auxjad.Phaser(wrong_type_input,  # noqa: F841
                               step_size=(1, 16),
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=62.3j,
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               max_steps='foo',
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               forward_bias='bar',
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               process_on_first_call='xyz',
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               remove_ties_connecting_windows=17j,
                               )
    with pytest.raises(ValueError):
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               max_steps=-1,
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               forward_bias=-0.3,
                               )
        phaser = auxjad.Phaser(container,  # noqa: F841
                               step_size=(1, 16),
                               forward_bias=1.4,
                               )


def test_Phaser_09():
    container = abjad.Container(r"c'2 d'2")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           )
    notes = phaser.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'2
            d'2
            c'4.
            d'8
            ~
            d'4.
            c'8
            c'4
            d'2
            c'4
            c'8
            d'8
            ~
            d'4
            ~
            d'8
            c'4.
        }
        """
    )
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           remove_unterminated_ties=False,
                           )
    notes = phaser.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'2
            d'2
            c'4.
            d'8
            ~
            d'4.
            c'8
            ~
            c'4
            d'2
            c'4
            ~
            c'8
            d'8
            ~
            d'4
            ~
            d'8
            c'4.
            ~
        }
        """
    )


def test_Phaser_10():
    container = abjad.Container(r"<d' fs' a'>2 c'2")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           )
    notes = phaser.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <d' fs' a'>2
            c'2
            <d' fs' a'>4.
            c'8
            ~
            c'4.
            <d' fs' a'>8
            <d' fs' a'>4
            c'2
            <d' fs' a'>4
            <d' fs' a'>8
            c'8
            ~
            c'4
            ~
            c'8
            <d' fs' a'>4.
        }
        """
    )
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           remove_unterminated_ties=False,
                           )
    notes = phaser.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <d' fs' a'>2
            c'2
            <d' fs' a'>4.
            c'8
            ~
            c'4.
            <d' fs' a'>8
            ~
            <d' fs' a'>4
            c'2
            <d' fs' a'>4
            ~
            <d' fs' a'>8
            c'8
            ~
            c'4
            ~
            c'8
            <d' fs' a'>4.
            ~
        }
        """
    )


def test_Phaser_11():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 32),
                           )
    notes = phaser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            c'8..
            d'32
            ~
            d'8..
            e'32
            ~
            e'8..
            f'32
            ~
            f'8..
            c'32
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )


def test_Phaser_12():
    container = abjad.Container(r"\time 3/8 c'8 d'8 e'8")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    notes = phaser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'8
            d'8
            e'8
            c'16
            d'8
            e'8
            c'16
            d'8
            e'8
            c'8
        }
        """
    )
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           forward_bias=0.0,
                           )
    notes = phaser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'8
            d'8
            e'8
            e'16
            c'8
            d'8
            e'16
            e'8
            c'8
            d'8
        }
        """
    )


def test_Phaser_13():
    container = abjad.Container(r"\time 3/8 c'8 d'8 e'8")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           forward_bias=0.0,
                           )
    notes = phaser.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'8
            d'8
            e'8
            e'8
            c'8
            d'8
            d'8
            e'8
            c'8
            c'8
            d'8
            e'8
        }
        """
    )


def test_Phaser_14():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           process_on_first_call=True,
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )


def test_Phaser_15():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           process_on_first_call=True,
                           remove_unterminated_ties=False,
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
            ~
        }
        """
    )


def test_Phaser_16():
    container = abjad.Container(r"c'4-.\p\< d'4--\f e'4->\p f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           )
    notes = phaser.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            \p
            - \staccato
            \<
            d'4
            \f
            - \tenuto
            e'4
            \p
            - \accent
            f'4
            c'8
            - \staccato
            \<
            d'4
            \f
            - \tenuto
            e'8
            \p
            - \accent
            ~
            e'8
            f'4
            c'8
            - \staccato
            d'4
            \f
            - \tenuto
            e'4
            \p
            - \accent
            f'4
            c'4
            - \staccato
            d'8
            \f
            - \tenuto
            e'4
            \p
            - \accent
            f'8
            ~
            f'8
            c'4
            - \staccato
            d'8
            \f
            - \tenuto
            e'4
            \p
            - \accent
            f'4
            c'4
            - \staccato
            \<
            d'4
            \f
            - \tenuto
        }
        """
    )


def test_Phaser_17():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )
    phaser.contents = abjad.Container(r"c'16 d'16 e'16 f'16 g'2.")
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'16
            d'16
            e'16
            f'16
            g'2.
        }
        """
    )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'16
            e'16
            f'16
            g'16
            ~
            g'2
            ~
            g'8.
            c'16
        }
        """
    )


def test_Phaser_18():
    container = abjad.Container(r"c'1")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    assert len(phaser) == 16
    container = abjad.Container(r"c'1 d'1 e'1 f'1")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    assert len(phaser) == 64
    container = abjad.Container(r"c'1")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    assert len(phaser) == 4
    container = abjad.Container(r"\time 3/4 c'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    assert len(phaser) == 3
    container = abjad.Container(r"\time 3/4 c'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 2),
                           )
    assert len(phaser) == 3


def test_Phaser_19():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           omit_time_signatures=True,
                           )
    notes = phaser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            c'8
            d'4
            e'4
            c'8
            d'4
            e'4
            c'4
        }
        """
    )


def test_Phaser_20():
    random.seed(98451)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           forward_bias=0.5,
                           )
    notes = phaser.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            d'4
            e'4
            f'4
            c'4
            c'4
            d'4
            e'4
            f'4
            f'4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Phaser_21():
    random.seed(12365)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           max_steps=4,
                           )
    notes = phaser.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            c''8
            f'8
            g'8
            a'8
            b'8
            c''8
            c'8
            d'8
            e'8
            c''8
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            f'8
            g'8
            a'8
            b'8
            c''8
            c'8
            d'8
            e'8
            a'8
            b'8
            c''8
            c'8
            d'8
            e'8
            f'8
            g'8
        }
        """
    )


def test_Phaser_22():
    container = abjad.Container(r"c'4. d'8 e'2")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'2
        }
        """
    )
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           boundary_depth=1,
                           )
    notes = phaser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            e'2
        }
        """
    )


def test_Phaser_23():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 ~ e'2.")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 4),
                           )
    staff = abjad.Staff()
    for window in phaser:
        staff.append(window)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            ~
            e'2.
            \time 3/4
            d'4
            e'2
            ~
            e'2
            c'4
            \time 3/4
            e'2.
            ~
            e'4
            c'4
            d'4
            \time 3/4
            e'2.
            c'4
            d'4
            e'4
            \time 3/4
            e'2
            c'4
            d'4
            e'2
            \time 3/4
            e'4
            c'4
            d'4
            e'2.
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            ~
            e'2.
            d'4
            e'2
            ~
            e'2
            c'4
            e'2.
            ~
            e'4
            c'4
            d'4
            e'2.
            c'4
            d'4
            e'4
            e'2
            c'4
            d'4
            e'2
            e'4
            c'4
            d'4
            e'2.
        }
        """
    )


def test_Phaser_24():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    phaser = auxjad.Phaser(tuplet, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(voice, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(staff, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    phaser = auxjad.Phaser(score, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    phaser = auxjad.Phaser(staff, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    phaser = auxjad.Phaser(score, step_size=(1, 16))
    assert isinstance(phaser(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        phaser = auxjad.Phaser(staff, step_size=(1, 16))  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        phaser = auxjad.Phaser(score, step_size=(1, 16))  # noqa: F841


def test_Phaser_25():
    container = abjad.Container(r"c'2(\p\< d'4. e'8\f f'4\p\> g'2 a'4\pp)")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 16),
                           )
    notes = phaser.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'2
            \p
            \<
            (
            d'4.
            e'8
            \f
            f'4
            \p
            \>
            g'2
            a'4
            \pp
            )
            c'4..
            \p
            \<
            (
            d'16
            ~
            d'4
            ~
            d'16
            e'8
            \f
            f'16
            \p
            ~
            f'8.
            \>
            g'16
            ~
            g'4
            ~
            g'8.
            a'16
            \pp
            ~
            a'8.
            )
            c'16
            \p
            (
            c'4.
            \<
            d'8
            ~
            d'4
            e'8
            \f
            f'8
            \p
            ~
            f'8
            \>
            g'8
            ~
            g'4
            ~
            g'8
            a'4
            \pp
            )
            c'8
            \p
            (
            c'4
            ~
            c'16
            \<
            d'8.
            ~
            d'8.
            e'16
            \f
            ~
            e'16
            f'8.
            \p
            ~
            f'16
            \>
            g'8.
            ~
            g'4
            ~
            g'16
            a'8.
            \pp
            ~
            a'16
            )
            c'8.
            \p
            (
            c'4
            \<
            d'4
            ~
            d'8
            e'8
            \f
            f'4
            \p
            \>
            g'2
            a'4
            \pp
            )
            c'4
            \p
        }
        """
    )


def test_Phaser_26():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 32),
                           )
    notes = phaser.output_n(3, tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            c'8..
            d'32
            ~
            d'8..
            e'32
            ~
            e'8..
            f'32
            ~
            f'8..
            c'32
            ~
            c'8.
            d'16
            ~
            d'8.
            e'16
            ~
            e'8.
            f'16
            ~
            f'8.
            c'16
        }
        """
    )


def test_Phaser_27():
    container = abjad.Container(r"c'2 d'2")
    phaser = auxjad.Phaser(container,
                           step_size=(1, 8),
                           )
    notes = phaser.output_all(tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'2
            d'2
            c'4.
            d'8
            ~
            d'4.
            c'8
            ~
            c'4
            d'2
            c'4
            ~
            c'8
            d'8
            ~
            d'4
            ~
            d'8
            c'4.
            d'2
            c'2
            d'4.
            c'8
            ~
            c'4.
            d'8
            ~
            d'4
            c'2
            d'4
            ~
            d'8
            c'8
            ~
            c'4
            ~
            c'8
            d'4.
            c'2
            d'2
        }
        """
    )



import abjad
import pytest

import auxjad


def test_extend_notes_01():
    staff = abjad.Staff(r"c'16 r2... d'8 r2.. e'8. r16 r2. f'4 r2.")
    auxjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )


def test_extend_notes_02():
    staff = abjad.Staff(
        r"\time 3/4 c'8 r8 r2 r4 <d' e' f'>4 r4 r8 g'16 r16 r2"
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((2, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2
            r4
            r4
            <d' e' f'>2
            r8
            g'8
            ~
            g'4.
            r8
        }
        """
    )


def test_extend_notes_03():
    staff = abjad.Staff(r"c'16 r2... d'16 r2... e'16 r2... f'16 r2...")
    auxjad.mutate.extend_notes(staff, (1, 4))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )
    staff = abjad.Staff(r"c'16 r2... d'16 r2... e'16 r2... f'16 r2...")
    auxjad.mutate.extend_notes(staff, 0.25)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )
    staff = abjad.Staff(r"c'16 r2... d'16 r2... e'16 r2... f'16 r2...")
    auxjad.mutate.extend_notes(staff, 1 / 4)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )
    staff = abjad.Staff(r"c'16 r2... d'16 r2... e'16 r2... f'16 r2...")
    auxjad.mutate.extend_notes(staff, "1/4")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )


def test_extend_notes_04():
    staff = abjad.Staff(r"c'16 r4.. d'16 r4.. e'16 r2... f'16 r4.. g'16 r4..")
    auxjad.mutate.extend_notes(staff, abjad.Duration((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            d'2
            e'2.
            r4
            f'2
            g'2
        }
        """
    )


def test_extend_notes_05():
    staff = abjad.Staff(
        r"c'16\ppp r2... d'16\ff r2... e'16\f r2... f'16\mp r2..."
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2.
            \ppp
            r4
            d'2.
            \ff
            r4
            e'2.
            \f
            r4
            f'2.
            \mp
            r4
        }
        """
    )


def test_extend_notes_06():
    staff = abjad.Staff(
        r"r16 c'16 r4. r16 d'16 r4. r16 e'16 r4. r16 f'16 r4."
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r16
            c'8.
            ~
            c'16
            r8.
            r16
            d'8.
            ~
            d'16
            r8.
            r16
            e'8.
            ~
            e'16
            r8.
            r16
            f'8.
            ~
            f'16
            r8.
        }
        """
    )
    staff = abjad.Staff(
        r"r16. c'32 r4. r16. d'32 r4. r16. e'32 r4. r16. f'32 r4."
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r16.
            c'32
            ~
            c'8
            ~
            c'16.
            r32
            r8
            r16.
            d'32
            ~
            d'8
            ~
            d'16.
            r32
            r8
            r16.
            e'32
            ~
            e'8
            ~
            e'16.
            r32
            r8
            r16.
            f'32
            ~
            f'8
            ~
            f'16.
            r32
            r8
        }
        """
    )


def test_extend_notes_07():
    staff = abjad.Staff(
        r"\time 3/4 c'16 r8. r2 \time 2/4 d'8 r8 e'8 r8 \time 3/4 r2 f'16 r8."
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            \time 2/4
            d'4
            e'4
            ~
            \time 3/4
            e'2
            f'4
        }
        """
    )


def test_extend_notes_08():
    staff = abjad.Staff(r"c'16 r4.. d'16 r4..")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((1, 4)),
                               rewrite_meter=False,
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            ~
            c'8.
            r4
            d'16
            ~
            d'8.
            r4
        }
        """
    )


def test_extend_notes_09():
    staff = abjad.Staff(r"c'16 r2... d'16 r2... e'16 r2... f'16 r2...")
    abjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'4
            r2.
            e'4
            r2.
            f'4
            r2.
        }
        """
    )


def test_extend_notes_10():
    staff = abjad.Staff(r"c'16 r2... d'2 r2 e'2. r4 f'1")
    auxjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r2.
            d'2
            r2
            e'2.
            r4
            f'1
        }
        """
    )


def test_extend_notes_11():
    staff = abjad.Staff(
        r"r8 c'8 r2. r8 "
        r"d'8 ~ d'4 r2 "
        r"r16 e'8. ~ e'4 r2 "
        r"r16 f'8. ~ f'2."
    )
    auxjad.mutate.extend_notes(staff, abjad.Duration((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r8
            c'8
            ~
            c'2
            ~
            c'8
            r8
            r8
            d'8
            ~
            d'2
            ~
            d'8
            r8
            r16
            e'8.
            ~
            e'2
            ~
            e'16
            r8.
            r16
            f'2...
        }
        """
    )


def test_extend_notes_12():
    staff = abjad.Staff(r"r4 c'4 r4 c'4 r8 c'8 r2")
    auxjad.mutate.extend_notes(staff, abjad.Duration((2, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r4
            c'2
            c'4
            ~
            c'8
            c'8
            ~
            c'4
            ~
            c'8
            r4.
        }
        """
    )


def test_extend_notes_13():
    staff = abjad.Staff(r"\time 3/4 r8 c'8 r4 c'4 R1 * 3/4 r8 c'8 r2 R1 * 3/4")
    auxjad.mutate.extend_notes(staff, abjad.Duration((2, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            r8
            c'4.
            c'4
            ~
            c'4
            r2
            r8
            c'8
            ~
            c'4.
            r8
            R1 * 3/4
        }
        """
    )
    staff = abjad.Staff(r"\time 3/4 r8 c'8 r4 c'4 r2. r8 c'8 r2 r2.")
    auxjad.mutate.extend_notes(staff, abjad.Duration((2, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            r8
            c'4.
            c'4
            ~
            c'4
            r2
            r8
            c'8
            ~
            c'4.
            r8
            R1 * 3/4
        }
        """
    )
    staff = abjad.Staff(r"\time 3/4 r8 c'8 r4 c'4 R1 * 3/4 r8 c'8 r2 R1 * 3/4")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((2, 4)),
                               use_multimeasure_rests=False,
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            r8
            c'4.
            c'4
            ~
            c'4
            r2
            r8
            c'8
            ~
            c'4.
            r8
            r2.
        }
        """
    )


def test_extend_notes_14():
    staff = abjad.Staff(r"c'4 r4 d'4 r4 e'4 r2.")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((2, 4)),
                               gap=abjad.Duration((1, 16)),
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4..
            r16
            d'4..
            r16
            e'2
            r2
        }
        """
    )


def test_extend_notes_15():
    staff = abjad.Staff(r"c'2 d'4 r4 e'4 r4 f'2 ~ f'2 r2")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((2, 4)),
                               gap=abjad.Duration((1, 16)),
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            d'4..
            r16
            e'4..
            r16
            f'2
            ~
            f'2
            r2
        }
        """
    )


def test_extend_notes_16():
    staff = abjad.Staff(r"c'16 r8. d'16 r8. e'16 r8. f'16 r8.")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((1, 4)),
                               gap=abjad.Duration((1, 16)),
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'8.
            r16
            d'8.
            r16
            e'8.
            r16
            f'4
        }
        """
    )
    staff = abjad.Staff(r"c'16 r8. d'16 r8. e'16 r8. f'16 r8.")
    auxjad.mutate.extend_notes(staff,
                               abjad.Duration((1, 4)),
                               gap=abjad.Duration((1, 16)),
                               gap_before_end=True,
                               )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'8.
            r16
            d'8.
            r16
            e'8.
            r16
            f'8.
            r16
        }
        """
    )


def test_extend_notes_17():
    staff = abjad.Staff(r"c'4 r4 \times 2/3 {r4 d'4 r4} e'4 r2.")
    with pytest.raises(ValueError):
        auxjad.mutate.extend_notes(staff, abjad.Duration((1, 4)))
